/**
* ppunpack.dll - PP (powerpacker) adatallomanyt tamogato rutinok
*
* @Author      : Peter Bakota, e-mail: bakota@tippnet.co.yu
* @Last update : 07.09.2001.
*
*/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

// PP data support header
#include "ppdata.h"
#include "ppunpack.h"

#ifdef __cplusplus
extern "C" {
#endif

// Ezzel a modszerrel csak MAX. 16MB hosszusag merheto mivel a PP a legalso bajtot sajat celjaira hasznalja!
#define val(x) ((unsigned long)(*(char *)(x+0)) << 16 | (unsigned long)(*(char *)(x+1)) << 8 | (unsigned long)(*(char *)(x+2)))

// API
//unsigned long WINAPI ppLoadFile(char *FileName, char **Buffer, unsigned long *uLen);
//void		  WINAPI ppFreeBuffer(char *Buffer, unsigned long uLen);
//void          WINAPI ppDecrunchBuffer(unsigned char *Buffer, unsigned char *UBuffer, unsigned long pLen, unsigned long uLen);
//bool          WINAPI ppGetUSize(char *Buffer, unsigned long pLen, unsigned long *uLen);
//bool          WINAPI ppGetUSizeFromFile(char *FileName, unsigned long *uLen);
//unsigned long WINAPI ppDecrunchFile(char *Src, char *Dest);
//bool		  WINAPI ppIsPPFile(char *FileName);

// private functions prototypes
static bool IsPPData(char *);
static unsigned long fgetlong3(FILE *);
static bool IsPPFile(FILE *);

/**
* Egy PP allomany beolvasasa a memoriaba valamint automatikus szetpakolasa
* Belepeskor a memoria pointer tarolojanak kezdocimet, valamint a szetpakolt hooszusag tarolojanak kezdocimet kell megadni!
* Kilepeskor TRUE ha a muvelet sikeres volt FALSE ha nem
*/
unsigned long WINAPI ppLoadFile(char *FileName, char **Buffer, unsigned long *uLen)
	{
	FILE *fp;
	unsigned long upLen;
	unsigned long ppLen;
	char *ppBuf;
	char *upBuf;

	// Megnyitjuk az allomanyt
	if ((fp=fopen(FileName, "rb"))==NULL) return PP_OPENERR;
	// Ellenorizzuk hogy valoban PP allomanyrol van -e szo?
	if (!IsPPFile( fp ))
		{
		fclose(fp);
		return PP_NOTPPDATA;
		}

	fseek(fp, 0, SEEK_END);
	ppLen = ftell( fp );
	if ((ppBuf = (char *)malloc((size_t)ppLen))==NULL)
		{
		fclose(fp);
		return PP_NOMEMORY;
		}

	fseek(fp, -4, SEEK_CUR);
	upLen = fgetlong3( fp );
	if ((upBuf = (char *)malloc((size_t)upLen))==NULL)
		{
		free(ppBuf);
		fclose(fp);
		return PP_NOMEMORY;
		}

	fseek(fp, 0, SEEK_SET);
	if (fread(ppBuf, ppLen, 1, fp)!=1)
		{
		free(ppBuf);
		free(upBuf);
		fclose(fp);
		return PP_READERR;
		}

	ppDecrunchBuffer((unsigned char *)ppBuf, (unsigned char *)upBuf, ppLen, upLen);
	fclose(fp);
	free(ppBuf);

	// Ha minden OK akkor beallitjuk a visszateresi paramatereket!
	*Buffer = upBuf;
	*uLen = upLen;
	return PP_LOADOK;

	}

/**
* A ppLoadFile altal lefoglat memoria felszabaditasa
* Az uLen paramteter csak az esetleges jovobeli bovites megkonnyitese vegett van!
*/ 
void WINAPI ppFreeBuffer(char *Buffer, unsigned long uLen)
	{
	free( Buffer );
	}

/**
* Egy PP memoria adathalmaz szetpakolasa
*/

/*

         *********************************************************
         ****         VEGA - PowerPacker Unpacker V1.0        ****
         ****        Original code by Marc Espie (1995)       ****
         **** Original PowerPacker algorithm by Nico Francois ****
         ****      Adapted to VEGA by Alessandro Pedretti     ****
		 ****      Adapted to ppunapack.dll by Peter Bakota   ****
         *********************************************************


*/

typedef unsigned long ULONG;
typedef unsigned char UBYTE;

/**** Local variables ****/

static ULONG    counter;
static ULONG    shift_in;
static UBYTE    *source;

/**** Get a sequence o bits ****/

static ULONG get_bits(ULONG n)
{
  register ULONG        result = 0;
  register ULONG        i;

  for (i = 0; i < n; i++) {
    if (counter == 0) {
      counter = 8;
      shift_in = *--source;
    }
    result = (result<<1) | (shift_in & 1);
    shift_in >>= 1;
    counter--;
  }

  return result;
}

/**** Depack a segment of memory ****/

void WINAPI ppDecrunchBuffer(unsigned char *packed, unsigned char *depacked, unsigned long plen, unsigned long unplen)
{
  register UBYTE        *dest;
  register int          n_bits;
  register int          idx;
  register ULONG        bytes;
  register int          to_add;
  register ULONG        offset;
  UBYTE                 offset_sizes[4];
  register ULONG        i;

  /**** Skip signature ****/

  offset_sizes[0] = packed[4];
  offset_sizes[1] = packed[5];
  offset_sizes[2] = packed[6];
  offset_sizes[3] = packed[7];

  /**** Initialize source of bits ****/

  source = packed + plen - 4;
  dest   = depacked + unplen;

  counter = shift_in = 0; // Clear variables

  /**** Skip bits ****/

  get_bits(source[3]);

  /**** Do it forever, i.e., while the whole file isn't unpacked ****/

  while (1) {
    /**** Copy some bytes from the source anyway ****/

    if (get_bits(1) == 0) {
      bytes = 0;
      do {
        to_add = get_bits(2);
        bytes += to_add;
      } while (to_add == 3);
      for (i = 0; i <= bytes; i++)
        *--dest = (unsigned char )get_bits(8);
      if (dest <= depacked) return;
    }

    /**** Decode what to copy from the destination file ****/

    idx    = get_bits(2);
    n_bits = offset_sizes[idx];

    /**** Bytes to copy ****/

    bytes = idx + 1;
    if (bytes == 4) {	/* 4 means >=4               */
                        /* and maybe a bigger offset */
      if (get_bits(1) == 0)
        offset = get_bits(7);
      else
        offset = get_bits(n_bits);

      do {
        to_add = get_bits(3);
        bytes += to_add;
      } while (to_add == 7);
    } else offset = get_bits(n_bits);
    for (i = 0; i <= bytes; i++) {
      dest[-1] = dest[offset];
      dest--;
    }
    if (dest <= depacked) return;
  }
}
/**
* END OF PP DEPACK ROUTINE
*/


/**
* A PP adat szetpakolt hosszanak a lekerdezese a memoria pointerbol
* Visszatereskor TRUE ha PP adat FALSE ha nem!
*/
int __stdcall ppGetUSize(char *Buffer, unsigned long pLen, unsigned long *uLen)
	{
	register char *p = Buffer;
	if (!IsPPData(Buffer)) return FALSE;
	p += pLen - 4;
	*uLen = val( p );
	return TRUE;
	}

/**
* A PP adat szetpakolt hosszanak lekerdezese egy allomanybol!
* Visszatereskor TRUE ha PP allomany FALSE ha nem!
*/
int __stdcall ppGetUSizeFromFile(char *FileName, unsigned long *uLen)
	{
	FILE *fp;
	if ((fp = fopen(FileName, "rb"))==NULL) return FALSE;
	if (!IsPPFile(fp))
		{
		fclose(fp);
		return FALSE;
		}
	fseek(fp, -4, SEEK_END);
	*uLen = fgetlong3( fp );
	fclose(fp);
	return TRUE;	
	}
/**
* Ellenorzi hogy a megadott adat kezdet valoban PP adatotkat tartalmaz-e?
* A mutato kilepeskor kozvetlenul a 'PP20' id utanra pozicionalt ha PP adatrol van szo,
* ha pedig nem akkor nem definialt!
*/
bool IsPPData(char *ptr)
	{
	return( *ptr++=='P' && *ptr++=='P' && *ptr++=='2' && *ptr++=='0' );
	}

/**
* Hasonloan mint az elozo rutin csak most a tarolon levo allomanybol kerdezzuk le.
* A FILEPOINTER az ID utanra pozicionalt helyes adat eseteben!
*/
bool IsPPFile(FILE *fp)
	{
	return( fgetc(fp)=='P' && fgetc(fp)=='P' && fgetc(fp)=='2' && fgetc(fp)=='0' );
	}

/**
* Ellenorzi hogy a megadott allomany PP allomany-e?
*/
int __stdcall ppIsPPFile(char *src)
	{
	FILE *fp;
	bool res;
	if((fp=fopen(src, "rb"))==NULL)	return FALSE;
	res = IsPPFile( fp );
	fclose( fp );
	return res;
	}

/**
* Egy (majdnem)hosszu szo beolvasasa az allomanybol!
* Azert "majdnem" mert csak 3 bajtot vesz figyelembe!
*/
unsigned long fgetlong3(FILE *fp)
	{
	return( (unsigned long)fgetc(fp) << 16 | (unsigned long)fgetc(fp) << 8 | (unsigned long)fgetc(fp) );
	}

/**
* Egy allomany szetpakolasa egy masik allomanyba!
*/
unsigned long WINAPI ppDecrunchFile(char *src, char *dest)
	{
	unsigned long upLen;
	char *upBuf;
	unsigned long res;
	if ((res=ppLoadFile(src, &upBuf, &upLen))==PP_LOADOK)
		{
		FILE *fp;
		if ((fp=fopen(dest, "wb"))==NULL)
			{
			ppFreeBuffer(upBuf, upLen);
			return PP_OPENERR;
			}
		if(fwrite(upBuf, upLen, 1, fp)!=1)
			{
			ppFreeBuffer(upBuf, upLen);
			fclose(fp);
			return PP_WRITEERR;
			}
		fclose(fp);
		ppFreeBuffer(upBuf, upLen);
		return PP_LOADOK;
		}
	else
		return res;
	}

#ifdef __cplusplus
	}
#endif
